/*
 * dev_clinet_api.h
 *
 *  Created on: Jan 25, 2021
 *  Copyright by Xinhuan Intelligence co,.ltd 2021
 */

#ifndef SRC_DEV_API_CLIENT_API_H_
#define SRC_DEV_API_CLIENT_API_H_

#include "hi_comm_isp.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* End of #ifdef __cplusplus */


const static int MAX_CODES = 12;
const static int MAX_PLATE_NO_BYTES = 64;

typedef struct PlateBox {
	float xmin;   // 左上角x轴坐标
	float ymin;   // 左上角y轴坐标
	float xmax;   // 右下角x轴坐标
	float ymax;   // 右下角y轴坐标
	float score;  // 车牌框置信度
} PlateBox;

typedef struct sSpeed {
	int value;
	int limitation;
	int status;
}Speed;

typedef struct sPlateBasicInfo {
	int label;			   				// 识别的目标类型，小于等于1是车牌，大于1其他目标
	float prob;            				// 车牌识别置信度
	PlateBox bbox;         				// 车牌位置
	Speed speed;						// 车速
	float confidences[MAX_CODES];  		// 车牌号码单字置信度数组
	int len;							// 车牌号码单字置信度数组长度
	char plateNos[MAX_PLATE_NO_BYTES];  // UTF8变长码编码的车牌号码。
	int plateNoBytes;     				// 车牌号码实际实际长度，不超过MAX_PLATE_NO_BYTES变量定义
	char plateColor[16];                //  白  黄  蓝  黑  绿  黄绿
	int plateColorBytes;				//  车牌颜色字节数
	int anchorBoxNo;					// 识别框编号，取值-1, 0, 1, 2,...，-1表示不支持此属性或者没有输出此属性

	/**
	 * 中国大陆车牌类型，定义如下
	 * 0 {未知}
	 * 10 - 16  {军 双层军牌 武警 双层武警牌 警 应急 新电动车}
	 * 20 - 23 {黄 教练车牌 双层黄牌 挂车车牌}
	 * 30 {蓝色小汽车}
	 * 40 - 43 {使馆牌 领馆牌 粤港澳牌 黑色小汽车 }
	 * 50 - 53  {民航 新能源 农用车牌 电动车牌（非机动）}
	 * 60 {大型双能源}
	 */
	int plateType ;
	char * imageBuffer;				    // 图片Buffer, JPEG格式
	int imageBufferSize;                //

	char * plateImageBuffer;
	int plateImageBufferSize;

	char deviceIP[15];

	/**
	 * 方向标志，32位，大端开头顺序，
	 * 第0位代表左右，0表示左边，1表示右边
	 * 第1位代表上下，0表示上部，1表示下部
	 *	+------------------------------+
	 *	|           保留           |Y|X|
	 *	+------------------------------+
	 */
	int direction;

	// 识别框里中车牌数量，默认0，识别成功情况 >=1
	int peerNum = 0;

} PlateBasicInfo;


typedef struct sDeviceInfo {
		char deviceName[64]; // 设备名称
		char ip[32];
		char firmware[16]; 	// 固件号
		char version[16];    // 版本
		char productNo[24] ; // 产品序列号
		int minVersion ;  // minor version
		unsigned long startup ;     // 设备启动时间戳
		char productType[16];       // 产品型号
} DeviceInfo;

enum API_STATUS {
	OK,
	FAILED
};

// typedef void (_stdcall  *API_CALLBACK)(PlateBasicInfo *);
typedef void (*API_CALLBACK)(PlateBasicInfo *);


/**
 ********* Basic working mechanism   ***********************************
 *
 * The license plate recognition HiSilicon module is referred to as
 * the module for short, and the module communicates with the client
 * through the TCP/IP protocol. In the process of client initializa-
 * tion, start the service of receiving license plate information.
 * After the module recognizes the license plate, it sends the lice-
 * nse plate recognition information to the client.
 *
 ***********************************************************************/

/**
 * initialize device communication API with same local IP, only support run on device.
 * return OK if success else FAILED
 */
API_STATUS init_local_client(int local_port, API_CALLBACK cb);

/**
 * initialize device communication API, return OK if success else FAILED
 */
API_STATUS init_client(const char * local_ip, int local_port, API_CALLBACK cb);

API_STATUS connect_device(const char * device_ip);
/**
 * close client api
 */
void close_client();


/***********************************************************************
 * below functions neither need initialize API nor close API
 ***********************************************************************/

/**
 * reboot device, device will reboot in 30 seconds
 */
void reboot_device(const char * device_ip);

/**
 * set anchor box for device via IP,
 * the xmin, ymin is the left-top vertex, value is between 0, 1
 * the xmax, ymax is the right-bottom vertex, value is between 0, 1
 * and with limitation: xmin < xmax, ymin < ymax
 */
API_STATUS set_anchor_box(const char * device_ip, float xmin, float ymin, float xmax, float ymax);

API_STATUS get_anchor_box(const char * device_ip, float* xmin, float* ymin, float* xmax, float* ymax);

/**
 *  change device login password, password DOES NOT exceeds 8 letter or number
 */
API_STATUS change_device_password(const char * device_ip, const char * cur_pwd, const char * new_pwd);

/**
 * get basic device information for device via IP, fill the DeviceInfo struct if return OK API_STATUS
 */
API_STATUS get_device_info(const char * device_ip, DeviceInfo * pDeviceInfo);

/**
 * force device to take a snapshot from video stream, and then post the snapshot to the client connected
 */
API_STATUS take_snapshot(const char * device_ip);

/**
 * switch WDR mode ON/OFF
 */
API_STATUS switch_wdrmode(const char * device_ip, const bool flag);

/**
 * rotate video with 0, 90, 180, 270 degree
 */
API_STATUS rotate_video(const char * device_ip, int r);

/**
 * set color luma for device
 */
API_STATUS set_luma(const char * device_ip, int luma);


/**
 * set RTC time, time format MMddhhmmyyyy.ss
 */
API_STATUS set_rtctime(const char * device_ip, const char * time_buffer, int len);

/**
 * get ISP exposure attributes, pass attributes to pointer of ISP_EXPOSURE_ATTR_S structure,
 * return OK API status when success
 */
API_STATUS get_isp_exposure_attr(const char * device_ip, ISP_EXPOSURE_ATTR_S * pDeviceInfo);

/**
 * set ISP exposure attributes with pointer parameter of ISP_EXPOSURE_ATTR_S structure,
 * return OK API status when success
 */
API_STATUS set_isp_exposure_attr(const char * device_ip, ISP_EXPOSURE_ATTR_S * pDeviceInfo);

/**
 * query ISP current exposure information, pass exposure statistic information to parameter of
 * ISP_EXP_INFO_S structure. Returns OK API status when success
 */
API_STATUS query_isp_exposure_info(const char * device_ip, ISP_EXP_INFO_S * pIspExpInfo);

#ifdef __cplusplus
#if __cplusplus
 }
#endif
#endif /* End of #ifdef __cplusplus */

#endif /* SRC_DEV_API_CLIENT_API_H_ */
